#ifndef HEADER_REAL
#define HEADER_REAL

#if LIBRARY_MODE == 0
	#include "iRRAM/lib.h"
	using namespace iRRAM;
	typedef REAL MyReal;
	#define MyReference int&

	inline MyReal initVar(int b) {
		REAL a = REAL(b);
		return a;
	}
	inline void init() {
		iRRAM_initialize(NULL, NULL);
	}

	inline int exec(int (*f)(const int& dummy)) {
		return iRRAM_exec(f, 0);
	}

	inline void finish() {}
	
	inline MyReal myAdd(MyReal a, MyReal b) {
		return (a + b);
	}
	inline MyReal mySub(MyReal a, MyReal b) {
		return (a - b);
	}
	inline MyReal myMul(MyReal a, MyReal b) {
		return (a * b);
	}
	inline MyReal myDiv(MyReal a, MyReal b) {
		return (a / b);
	}
	inline MyReal mySqrt(MyReal a) {
		return sqrt(a);
	}
	inline void myPrint(MyReal a, long long precision) {
		cout << setRwidth(precision + 8) << a;
	}
#elif LIBRARY_MODE == 1
	#include "RealLib/Real.h"
	#include <iostream>
	#include <iomanip>
	using namespace RealLib;
	using namespace std;

	typedef Real MyReal;
	#define MyReference int*

	inline MyReal initVar(int b) {
		Real a(b);
		return a;
	}
	inline void init() {
		InitializeRealLib();
	}

	inline int exec(int(*f)(const int* dummy)) {
		return f(NULL);
	}

	inline void finish() {
		FinalizeRealLib();
	}

	inline MyReal myAdd(MyReal a, MyReal b) {
		return (a + b);
	}
	inline MyReal mySub(MyReal a, MyReal b) {
		return (a - b);
	}
	inline MyReal myMul(MyReal a, MyReal b) {
		return (a * b);
	}
	inline MyReal myDiv(MyReal a, MyReal b) {
		return (a / b);
	}
	inline MyReal mySqrt(MyReal a) {
		return sqrt(a);
	}
	inline void myPrint(MyReal a, long long precision) {
		cout << setprecision(precision) << a;
	}

#elif LIBRARY_MODE == 2

	#include "xr/xr.h"
	typedef xr_t MyReal;
	#define MyReference int*

	inline MyReal initVar(int i) {
		xr_t a = xr_init(i, 1);
		return a;
	}

	inline void init() {}

	inline int exec(int (*f)(const int* dummy)) {
		return f(NULL);
	}
	inline void finish() {}

	inline MyReal myAdd(MyReal a, MyReal b) {
		return xr_add(a, b);
	}
	inline MyReal mySub(MyReal a, MyReal b) {
		return xr_sub(a, b);
	}
	inline MyReal myMul(MyReal a, MyReal b) {
		return xr_mul(a, b);
	}
	inline MyReal myDiv(MyReal a, MyReal b) {
		return xr_div(a ,b);
	}
	inline MyReal mySqrt(MyReal a) {
		return xr_sqrt(a);
	}
	inline void myPrint(MyReal a, long long precision) {
		xr_print(a, precision);
	}
#endif
#endif




